import ssl
from ldap3 import Connection, Server, ServerPool, SAFE_RESTARTABLE, Tls, AUTO_BIND_NONE
from perun.connector.utils.Logger import Logger
import time
import json


class LdapConnector:
    def __init__(self, config):
        self._logger = Logger.get_logger(self.__class__.__name__)
        self._servers = ServerPool()
        for server in config['servers']:
            self._servers.add(Server(server['hostname'],
                                     tls=Tls(validate=ssl.CERT_NONE)))

        self._enableTLS = False
        if config['start_tls'] == 'true':
            self._enableTLS = True

        self._user = config['username']
        self._password = config['password']
        self._conn = Connection(server=self._servers, auto_bind=AUTO_BIND_NONE,
                                user=self._user, password=self._password,
                                version=3, client_strategy=SAFE_RESTARTABLE,
                                read_only=True)
        if not self._conn:
            raise Exception("Unable to connect to the Perun LDAP,")

        hostname = self._servers.get_current_server(self._conn)
        # enable TLS if required
        if self._enableTLS and not str(hostname).startswith("ldaps:"):
            if not self._conn.start_tls():
                raise Exception('Unable to force STARTTLS on Perun LDAP')

    def search_for_entity(self, base, filters,
                          attr_names=None):
        entries = self._search(base, filters, attr_names)
        if not entries:
            self._logger.debug(f"ldap_connector.search_for_entity "
                               f"- No entity found. Returning \'None\'. "
                               f"query base: {base} "
                               f", filter: {filters} ")
            return None

        if len(entries) > 1:
            raise Exception(f"ldap_connector.search_for_entity - "
                            f"More than one entity found. query base:"
                            f"{base}, filter: {filters}. Hint: Use "
                            f"method search_for_entities if you expect "
                            f"array of entities.")

        return entries[0]

    def search_for_entities(self, base, filters,
                            attr_names=None):
        entries = self._search(base, filters, attr_names)

        if not entries:
            self._logger.debug(f"ldap_connector.search_for_entities - "
                               f"No entities found. Returning empty "
                               f"array. query base: {base} "
                               f"filter: {filters} ")

            return entries

        return entries

    def _search(self, base, filters, attributes=None):
        hostname = self._servers.get_current_server(self._conn)
        if not self._conn.bind():
            raise Exception('Unable to bind user to the Perun LDAP,' +
                            hostname)
        self._logger.debug(f"ldap_connector.search - Connection "
                           f"to Perun LDAP established. Ready to "
                           f"perform search query. host: "
                           f"{hostname}, user: {self._user}")

        start_time = time.time()
        status, result, response, _ = \
            self._conn.search(search_base=base, search_filter=filters,
                              attributes=attributes)
        end_time = time.time()

        response_time = round(end_time - start_time, 3)
        if not response:
            return []

        entries = self._get_simplified_entries(response)

        self._conn.unbind()

        self._logger.debug(f"ldap_connector.search - search query "
                           f"proceeded in {str(response_time)}"
                           f"ms. Query base: {base}, filter: "
                           f"{filters}, response: ' "
                           f"{json.dumps(str(entries))}")

        return entries

    @staticmethod
    def _get_simplified_entries(result):

        entries = []
        for entry in result:
            entries.append(entry['attributes'])

        return entries



"""
    Perun RPC API

    Perun Remote Procedure Calls Application Programming Interface  # noqa: E501

    The version of the OpenAPI document: 0.0.0
    Contact: perun@cesnet.cz
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from perun.connector.perun_openapi.api_client import ApiClient, Endpoint as _Endpoint
from perun.connector.perun_openapi.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from perun.connector.perun_openapi.model.ext_source import ExtSource
from perun.connector.perun_openapi.model.ext_source_object import ExtSourceObject
from perun.connector.perun_openapi.model.perun_exception import PerunException


class ExtSourcesManagerApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.add_ext_source_with_group_source_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/addExtSource/g-s',
                'operation_id': 'add_ext_source_with_group_source',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'group',
                    'source',
                ],
                'required': [
                    'group',
                    'source',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'group':
                        (int,),
                    'source':
                        (int,),
                },
                'attribute_map': {
                    'group': 'group',
                    'source': 'source',
                },
                'location_map': {
                    'group': 'query',
                    'source': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.add_ext_source_with_vo_source_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/addExtSource/v-s',
                'operation_id': 'add_ext_source_with_vo_source',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'vo',
                    'source',
                ],
                'required': [
                    'vo',
                    'source',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'vo':
                        (int,),
                    'source':
                        (int,),
                },
                'attribute_map': {
                    'vo': 'vo',
                    'source': 'source',
                },
                'location_map': {
                    'vo': 'query',
                    'source': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.create_ext_source_from_ext_source_object_endpoint = _Endpoint(
            settings={
                'response_type': (ExtSource,),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/createExtSource/es',
                'operation_id': 'create_ext_source_from_ext_source_object',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'ext_source_object',
                ],
                'required': [
                    'ext_source_object',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'ext_source_object':
                        (ExtSourceObject,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'ext_source_object': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.create_ext_source_with_name_type_endpoint = _Endpoint(
            settings={
                'response_type': (ExtSource,),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/createExtSource/n-t',
                'operation_id': 'create_ext_source_with_name_type',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'name',
                    'type',
                ],
                'required': [
                    'name',
                    'type',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name':
                        (str,),
                    'type':
                        (str,),
                },
                'attribute_map': {
                    'name': 'name',
                    'type': 'type',
                },
                'location_map': {
                    'name': 'query',
                    'type': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.delete_ext_source_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/deleteExtSource',
                'operation_id': 'delete_ext_source',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_ext_source_by_id_endpoint = _Endpoint(
            settings={
                'response_type': (ExtSource,),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/getExtSourceById',
                'operation_id': 'get_ext_source_by_id',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_ext_source_by_name_endpoint = _Endpoint(
            settings={
                'response_type': (ExtSource,),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/getExtSourceByName',
                'operation_id': 'get_ext_source_by_name',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name',
                ],
                'required': [
                    'name',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name':
                        (str,),
                },
                'attribute_map': {
                    'name': 'name',
                },
                'location_map': {
                    'name': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_ext_sources_endpoint = _Endpoint(
            settings={
                'response_type': ([ExtSource],),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/getExtSources',
                'operation_id': 'get_ext_sources',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_group_ext_sources_endpoint = _Endpoint(
            settings={
                'response_type': ([ExtSource],),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/getGroupExtSources',
                'operation_id': 'get_group_ext_sources',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'group',
                ],
                'required': [
                    'group',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'group':
                        (int,),
                },
                'attribute_map': {
                    'group': 'group',
                },
                'location_map': {
                    'group': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_vo_ext_sources_endpoint = _Endpoint(
            settings={
                'response_type': ([ExtSource],),
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/getVoExtSources',
                'operation_id': 'get_vo_ext_sources',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'vo',
                ],
                'required': [
                    'vo',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'vo':
                        (int,),
                },
                'attribute_map': {
                    'vo': 'vo',
                },
                'location_map': {
                    'vo': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.load_ext_sources_definitions_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/json/extSourcesManager/loadExtSourcesDefinitions',
                'operation_id': 'load_ext_sources_definitions',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.remove_ext_source_with_group_source_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/removeExtSource/g-s',
                'operation_id': 'remove_ext_source_with_group_source',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'group',
                    'source',
                ],
                'required': [
                    'group',
                    'source',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'group':
                        (int,),
                    'source':
                        (int,),
                },
                'attribute_map': {
                    'group': 'group',
                    'source': 'source',
                },
                'location_map': {
                    'group': 'query',
                    'source': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.remove_ext_source_with_vo_source_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'ApiKeyAuth',
                    'BasicAuth',
                    'BearerAuth'
                ],
                'endpoint_path': '/urlinjsonout/extSourcesManager/removeExtSource/v-s',
                'operation_id': 'remove_ext_source_with_vo_source',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'vo',
                    'source',
                ],
                'required': [
                    'vo',
                    'source',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'vo':
                        (int,),
                    'source':
                        (int,),
                },
                'attribute_map': {
                    'vo': 'vo',
                    'source': 'source',
                },
                'location_map': {
                    'vo': 'query',
                    'source': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def add_ext_source_with_group_source(
        self,
        group,
        source,
        **kwargs
    ):
        """Associate an external source definition with a GROUP.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.add_ext_source_with_group_source(group, source, async_req=True)
        >>> result = thread.get()

        Args:
            group (int): id of Group
            source (int): external source id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['group'] = \
            group
        kwargs['source'] = \
            source
        return self.add_ext_source_with_group_source_endpoint.call_with_http_info(**kwargs)

    def add_ext_source_with_vo_source(
        self,
        vo,
        source,
        **kwargs
    ):
        """Associate an external source definition with a VO.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.add_ext_source_with_vo_source(vo, source, async_req=True)
        >>> result = thread.get()

        Args:
            vo (int): id of Vo
            source (int): external source id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['vo'] = \
            vo
        kwargs['source'] = \
            source
        return self.add_ext_source_with_vo_source_endpoint.call_with_http_info(**kwargs)

    def create_ext_source_from_ext_source_object(
        self,
        ext_source_object,
        **kwargs
    ):
        """Creates an external source. ExtSource object must contain: name, type. Other parameters are ignored.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_ext_source_from_ext_source_object(ext_source_object, async_req=True)
        >>> result = thread.get()

        Args:
            ext_source_object (ExtSourceObject):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            ExtSource
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['ext_source_object'] = \
            ext_source_object
        return self.create_ext_source_from_ext_source_object_endpoint.call_with_http_info(**kwargs)

    def create_ext_source_with_name_type(
        self,
        name,
        type,
        **kwargs
    ):
        """Creates an external source.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_ext_source_with_name_type(name, type, async_req=True)
        >>> result = thread.get()

        Args:
            name (str): name of entity
            type (str): type of ExtSource

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            ExtSource
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['name'] = \
            name
        kwargs['type'] = \
            type
        return self.create_ext_source_with_name_type_endpoint.call_with_http_info(**kwargs)

    def delete_ext_source(
        self,
        id,
        **kwargs
    ):
        """Delete an external source.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_ext_source(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): numeric id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['id'] = \
            id
        return self.delete_ext_source_endpoint.call_with_http_info(**kwargs)

    def get_ext_source_by_id(
        self,
        id,
        **kwargs
    ):
        """Returns an external source by its id.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_ext_source_by_id(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): numeric id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            ExtSource
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['id'] = \
            id
        return self.get_ext_source_by_id_endpoint.call_with_http_info(**kwargs)

    def get_ext_source_by_name(
        self,
        name,
        **kwargs
    ):
        """Returns an external source by its name.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_ext_source_by_name(name, async_req=True)
        >>> result = thread.get()

        Args:
            name (str): name of entity

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            ExtSource
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['name'] = \
            name
        return self.get_ext_source_by_name_endpoint.call_with_http_info(**kwargs)

    def get_ext_sources(
        self,
        **kwargs
    ):
        """Returns the list of all external sources.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_ext_sources(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            [ExtSource]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        return self.get_ext_sources_endpoint.call_with_http_info(**kwargs)

    def get_group_ext_sources(
        self,
        group,
        **kwargs
    ):
        """Returns the list of external sources associated with a GROUP.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_group_ext_sources(group, async_req=True)
        >>> result = thread.get()

        Args:
            group (int): id of Group

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            [ExtSource]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['group'] = \
            group
        return self.get_group_ext_sources_endpoint.call_with_http_info(**kwargs)

    def get_vo_ext_sources(
        self,
        vo,
        **kwargs
    ):
        """Returns the list of external sources associated with a VO.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_vo_ext_sources(vo, async_req=True)
        >>> result = thread.get()

        Args:
            vo (int): id of Vo

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            [ExtSource]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['vo'] = \
            vo
        return self.get_vo_ext_sources_endpoint.call_with_http_info(**kwargs)

    def load_ext_sources_definitions(
        self,
        **kwargs
    ):
        """Loads ext source definitions from the configuration file and updates entries stored in the DB.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.load_ext_sources_definitions(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        return self.load_ext_sources_definitions_endpoint.call_with_http_info(**kwargs)

    def remove_ext_source_with_group_source(
        self,
        group,
        source,
        **kwargs
    ):
        """Remove an association of an external source from a GROUP.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.remove_ext_source_with_group_source(group, source, async_req=True)
        >>> result = thread.get()

        Args:
            group (int): id of Group
            source (int): external source id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['group'] = \
            group
        kwargs['source'] = \
            source
        return self.remove_ext_source_with_group_source_endpoint.call_with_http_info(**kwargs)

    def remove_ext_source_with_vo_source(
        self,
        vo,
        source,
        **kwargs
    ):
        """Remove an association of an external source from a VO.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.remove_ext_source_with_vo_source(vo, source, async_req=True)
        >>> result = thread.get()

        Args:
            vo (int): id of Vo
            source (int): external source id

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['vo'] = \
            vo
        kwargs['source'] = \
            source
        return self.remove_ext_source_with_vo_source_endpoint.call_with_http_info(**kwargs)




from typing import List, Union, Optional

from perun.connector.connectors.LdapConnector import LdapConnector
from perun.connector.models.Facility import Facility
from perun.connector.models.Group import Group
from perun.connector.models.User import User
from perun.connector.models.UserExtSource import UserExtSource
from perun.connector.models.VO import VO
from perun.connector.models.MemberStatusEnum import MemberStatusEnum
from perun.connector.utils.AttributeUtils import AttributeUtils
from perun.connector.utils.Logger import Logger
from perun.connector.adapters.AdapterInterface import AdapterInterface


class AdapterSkipException(Exception):
    def __init__(self, message='Adapter not able to execute given action'):
        self.message = message
        super().__init__(self.message)


class LdapAdapter(AdapterInterface):

    def __init__(self, config, attrs_map):
        self._logger = Logger.get_logger(self.__class__.__name__)
        self._ldap_base = config['base_dn']
        self.connector = LdapConnector(config)
        self._attribute_utils = AttributeUtils(attrs_map)
        self._RP_ID_ATTR = "perunFacilityAttr_rpID"

    def get_perun_user(self, idp_id: str, uids: List[str]) -> Optional[User]:
        query = ''
        for uid in uids:
            query += '(eduPersonPrincipalNames=' + uid + ')'

        if query == '':
            return None

        user = self.connector.search_for_entity(
            'ou=People,' + self._ldap_base, '(|' + query + ')',
            ['perunUserId', 'displayName', 'cn', 'givenName',
             'sn', 'preferredMail', 'mail'])

        if not user:
            return user
        if user['displayName']:
            name = user['displayName']
        elif user['cn']:
            name = user['cn'][0]
        else:
            name = None

        return User(user['perunUserId'], name)

    def get_group_by_name(self, vo: Union[VO, int], name: str) -> Group:
        vo_id = AdapterInterface.get_object_id(vo)
        group = self.connector.search_for_entity(
            'perunVoId=' + str(vo_id) + ',' + self._ldap_base,
            '(&(objectClass=perunGroup)(perunUniqueGroupName=' +
            name + '))', ['perunGroupId', 'cn', 'perunUniqueGroupName',
                          'perunVoId', 'uuid', 'description']
        )
        if not group:
            raise Exception('Group with name: ' + name + ' in VO: ' + str(vo_id) +
                            ' does not exists in Perun LDAP.')

        return self._create_internal_representation_group(group)

    def get_vo(self, short_name=None, vo_id=None) -> Optional[VO]:
        if short_name:
            vo = self.connector.search_for_entity(
                self._ldap_base,
                '(&(objectClass=perunVo)(o=' + short_name + '))',
                ['perunVoId', 'o', 'description']
            )

            if not vo:
                raise Exception('Vo with name: ' + short_name +
                                ' does not exists in Perun LDAP.')
        else:
            vo = self.connector.search_for_entity(
                self._ldap_base,
                '(&(objectClass=perunVo)(perunVoId=' + str(vo_id) + '))',
                ['o', 'description']
            )
            if not vo:
                raise Exception('Vo with id: ' + str(vo_id) +
                                ' does not exists in Perun LDAP.')

        return VO(
            vo_id or int(vo['perunVoId']),
            vo['description'][0],
            vo['o'][0]
        )

    def get_member_groups(self, user: Union[User, int], vo: Union[VO, int]) -> List[Group]:
        user_id = AdapterInterface.get_object_id(user)
        vo_id = AdapterInterface.get_object_id(vo)
        user_with_membership = self.connector.search_for_entity(
            'perunUserId=' + str(user_id) + ',ou=People,' + self._ldap_base,
            '(objectClass=perunUser)', ['perunUserId', 'memberOf'])
        groups = []
        for group_dn in user_with_membership['memberOf']:
            group_vo_id = group_dn.split(',')[1].split('=', 2)[1]
            if group_vo_id != str(vo_id):
                continue
            group = self.connector.search_for_entity(
                group_dn, '(objectClass=perunGroup)',
                ['perunGroupId', 'cn', 'perunUniqueGroupName',
                 'perunVoId', 'uuid', 'description'])
            groups.append(
                self._create_internal_representation_group(group)
            )

        return groups

    def get_sp_groups_by_facility(self, facility: Union[Facility, int]) -> List[Group]:
        if not facility:
            return []
        facility_id = AdapterInterface.get_object_id(facility)
        resources = self.connector.search_for_entities(
            self._ldap_base,
            '(&(objectClass=perunResource)(perunFacilityDn=perunFacilityId=' +
            str(facility_id) + ',' + self._ldap_base + '))',
            ['perunResourceId', 'assignedGroupId', 'perunVoId']
        )
        groups = []
        unique_ids = []

        for resource in resources:
            if 'assignedGroupId' in resource:
                for group_id in resource['assignedGroupId']:
                    group = self.connector.search_for_entity(
                        'perunGroupId=' + group_id + ',perunVoId=' +
                        resource['perunVoId'] + ',' + self._ldap_base,
                        '(objectClass=perunGroup)',
                        ['perunGroupId', 'cn', 'perunUniqueGroupName',
                         'perunVoId', 'uuid', 'description']
                    )
                    if group['perunGroupId'] not in unique_ids:
                        groups.append(
                            self._create_internal_representation_group(group)
                        )

                        unique_ids.append(group['perunGroupId'])
        return groups

    def get_sp_groups_by_rp_id(self, rp_id: str) -> List[Group]:
        facility = self.get_facility_by_rp_identifier(rp_id)
        return self.get_sp_groups_by_facility(facility)

    def get_user_attributes(
            self, user: Union[User, int], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        user_id = AdapterInterface.get_object_id(user)
        return self.connector.search_for_entity(
            'perunUserId=' + str(user_id) + ',ou=People,' + self._ldap_base,
            '(objectClass=perunUser)',
            attr_names
        )

    def get_entityless_attribute(
            self, attr_name: str
    ) -> Union[str, Optional[int], bool, List[str], dict[str, str]]:
        raise AdapterSkipException()

    def get_vo_attributes(
            self, vo: Union[VO, int], attr_names: List[str]
    ) -> dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]:
        raise AdapterSkipException()

    def get_facility_attribute(
            self, facility: Union[Facility, int], attr_name: str
    ) -> Union[str, Optional[int], bool, List[str], dict[str, str]]:
        raise AdapterSkipException()

    def get_facility_by_rp_identifier(
            self,
            rp_identifier: str,
    ) -> Optional[Facility]:
        attr_name = \
            self._attribute_utils.get_ldap_attr_name(self._RP_ID_ATTR) \
            or "entityID"
        ldap_result = self.connector.search_for_entity(
            self._ldap_base,
            '(&(objectClass=perunFacility)(' + attr_name + '=' +
            rp_identifier + '))',
            ['perunFacilityId', 'cn', 'description']
        )
        if not ldap_result:
            self._logger.warning('perun:AdapterLdap: '
                                 'No facility with entityID \'' +
                                 rp_identifier + '\' found.')
            return

        return Facility(
            ldap_result['perunFacilityId'],
            ldap_result['cn'][0],
            ldap_result['description'][0],
            rp_identifier
        )

    def get_users_groups_on_facility(
            self, facility: Union[Facility, int], user: Union[User, int]
    ) -> List[Group]:

        if not facility:
            return []

        facility_id = AdapterInterface.get_object_id(facility)
        user_id = AdapterInterface.get_object_id(user)

        resources = self.connector.search_for_entities(
            self._ldap_base,
            '(&(objectClass=perunResource)(perunFacilityDn='
            'perunFacilityId=' +
            str(facility_id) + ',' + self._ldap_base + '))',
            ['perunResourceId']
        )

        self._logger.debug('Resources - ' + str(resources))

        if not resources:
            raise Exception('Service with spEntityId: ' + str(facility_id) +
                            ' hasn\'t assigned any resource.')
        resources_string = '(|'
        for resource in resources:
            resources_string += '(assignedToResourceId=' + \
                                resource['perunResourceId'] + ')'
        resources_string += ')'
        result_groups = []
        unique_ids = []
        groups = self.connector.search_for_entities(
            self._ldap_base,
            '(&(uniqueMember=perunUserId=' + str(user_id) + ', ou=People,' +
            self._ldap_base + ')' + resources_string + ')',
            ['perunGroupId', 'cn', 'perunUniqueGroupName',
             'perunVoId', 'uuid', 'description']
        )
        for group in groups:
            if group['perunGroupId'] not in unique_ids:
                result_groups.append(
                    self._create_internal_representation_group(group)
                )
                unique_ids.append(group['perunGroupId'])

        self._logger.debug('Groups - ' + str(result_groups))

        return result_groups

    def get_users_groups_on_facility_by_rp_id(self, rp_identifier: str, user: Union[User, int]):
        facility = self.get_facility_by_rp_identifier(rp_identifier)
        return self.get_users_groups_on_facility(facility, user)

    def get_facilities_by_attribute_value(self, attribute: dict[str, str]):
        raise AdapterSkipException()

    def get_facility_attributes(self, facility: Union[Facility, int], attr_names: List[str]):
        raise AdapterSkipException()

    def get_user_ext_source(self, ext_source_name: str, ext_source_login: str):
        raise AdapterSkipException()

    def update_user_ext_source_last_access(self, user_ext_source: Union[UserExtSource, int]):
        raise AdapterSkipException()

    def get_user_ext_source_attributes(self, user_ext_source: Union[UserExtSource, int], attr_names: List[str]):
        raise AdapterSkipException()

    def set_user_ext_source_attributes(self,
                                       user_ext_source: Union[UserExtSource, int],
                                       attributes: List[
                                           dict[str, Union[str, Optional[int], bool, List[str], dict[str, str]]]]
                                       ):
        raise AdapterSkipException()

    def get_member_status_by_user_and_vo(self, user: Union[User, int], vo: Union[VO, int]):
        user_id = AdapterInterface.get_object_id(user)
        vo_id = AdapterInterface.get_object_id(vo)
        group_id = self.connector.search_for_entity(
            self._ldap_base,
            '(&(objectClass=perunGroup)(cn=members)(perunVoId=' + str(vo_id) +
            ')(uniqueMember=perunUserId=' + str(user_id) + ', ou=People,' +
            self._ldap_base + '))',
            ['perunGroupId']
        )

        if not group_id:
            raise AdapterSkipException(
                "Member status is other than valid. Skipping to another adapter to get MemberStatus")

        return MemberStatusEnum.VALID

    def is_user_in_vo_by_short_name(self, user: Union[User, int], vo_short_name: str) -> bool:
        user_id = AdapterInterface.get_object_id(user)
        if not user_id:
            raise Exception('userId is empty')
        if vo_short_name == '':
            raise Exception('voShortName is empty')

        vo = self.get_vo(vo_short_name)
        if not vo:
            self._logger.debug('isUserInVo - No VO found, returning false')

            return False

        return MemberStatusEnum.VALID == self.get_member_status_by_user_and_vo(user, vo)

    def get_resource_capabilities_by_facility(
            self, facility: Union[Facility, int], user_groups: List[Union[Group, int]]
    ) -> List[str]:
        if not facility:
            return []

        facility_id = AdapterInterface.get_object_id(facility)
        resources = self.connector.search_for_entities(
            self._ldap_base,
            '(&(objectClass=perunResource)(perunFacilityDn=perunFacilityId=' +
            str(facility_id) + ',' + self._ldap_base + '))',
            ['capabilities', 'assignedGroupId']
        )

        user_groups_ids = []
        for user_group in user_groups:
            user_groups_ids.append(str(AdapterInterface.get_object_id(user_group)))

        resource_capabilities = []
        for resource in resources:
            if ('assignedGroupId' not in resource) or \
                    ('capabilities' not in resource):
                continue
            for group_id in resource['assignedGroupId']:
                if group_id in user_groups_ids:
                    for resource_capability in resource['capabilities']:
                        resource_capabilities.append(resource_capability)

                    break

        return resource_capabilities

    def get_resource_capabilities_by_rp_id(
            self, rp_identifier: str, user_groups: List[Union[Group, int]]
    ) -> List[str]:
        facility = self.get_facility_by_rp_identifier(rp_identifier)
        return self.get_resource_capabilities_by_facility(facility, user_groups)

    def get_facility_capabilities_by_facility(self, facility: Union[Facility, int]) -> List[str]:
        if facility is None:
            return []
        facility_id = AdapterInterface.get_object_id(facility)
        facility_capabilities = self.connector.search_for_entity(
            self._ldap_base,
            '(&(objectClass=perunFacility)(entityID=' +
            str(facility_id) + '))',
            ['capabilities']
        )

        if not facility_capabilities:
            return []

        return facility_capabilities['capabilities']

    def get_facility_capabilities_by_rp_id(self, rp_identifier: str) -> List[str]:
        facility = self.get_facility_by_rp_identifier(rp_identifier)
        return self.get_facility_capabilities_by_facility(facility)

    def _create_internal_representation_group(
            self, group: dict[str, str]
    ) -> Group:
        return Group(
            int(group['perunGroupId']),
            self.get_vo(vo_id=int(group['perunVoId'])),
            group['uuid'],
            group['cn'][0],
            group['perunUniqueGroupName'],
            group['description'][0] or ''
        )
